#define PART1
//#define PART2
//#define PART3

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Threading;
using Microsoft.Cloud.Media.SDK.Client;
using Microsoft.Cloud.Media.SDK.Client.BulkIngest;
using System.IO;

namespace MediaWorkflow
{

    static class Program
    {
#if PART3
        

        static void Main(string[] args)
        {
            // Part 1 - Specify an asset to upload
	        //          Connect to Azure Media Services
            string inputFilePath = @"C:\windows\Performance\WinSat\winsat.wmv";

            CloudMediaContext mediaContext = new CloudMediaContext(
                            new Uri("https://nimbustap.cloudapp.net/API/"),
                            "F7DF85E4-E142-4042-A417-161FE60D409C",
                            "7HCe8R6n",
                            "urn:Nimbus",
                            "https://nimbusvoddev.accesscontrol.windows.net");

            // Part 2 - Upload an asset
            //          Create a policy for the asset
            //          Link access policy to asset
            IAsset asset = mediaContext.Assets.Create(inputFilePath);
            // Because this is a single-file asset, get the name of first file. 
            string fileName = asset.Files[0].Name;

            IAccessPolicy readPolicy = mediaContext.AccessPolicies.Create(
                "CanReadFor30Minutes",
                TimeSpan.FromMinutes(30),
                AccessPermissions.Read);

            asset.AccessPolicies.Add(readPolicy);

            // Part 3 - Define a locator based on the read policy 
	        //            and expiration date
            //          Print the path for the locator you created
            //          Get the locator path uri
            //          Build the full path to the file associated 
	        //            with the locator.  
            ILocator locator = asset.Locators.Where(c => c.AccessPolicyId ==
                readPolicy.Id && c.ExpirationDateTime > DateTime.UtcNow).First();

            Console.WriteLine("Locator path: " + locator.Path);

            var uriBuilder = new UriBuilder(locator.Path);

            uriBuilder.Path += Path.AltDirectorySeparatorChar + fileName;

            Uri fullUrl = uriBuilder.Uri;

            // Part 4 - Print the full path to the file 
            Console.WriteLine("Full URL to file: " + fullUrl);

        }
        static void DownloadAsset(string assetId, string outputMediaFilesFolder)
        {

            // Part 1 - Connect to Azure Media Services
            //          Get an existing asset based on assetId passed in.
            CloudMediaContext mediaContext = new CloudMediaContext(
                            new Uri("https://nimbustap.cloudapp.net/API/"),
                            "F7DF85E4-E142-4042-A417-161FE60D409C",
                            "7HCe8R6n",
                            "urn:Nimbus",
                            "https://nimbusvoddev.accesscontrol.windows.net");
            IAsset asset = mediaContext.Assets.Where(a => a.Id ==
                assetId).SingleOrDefault();

            // Part 2 - If the asset exists, download the first file in the asset
            //          Download to outputMediaFilesFolder       
            if (asset != null)
            {
                Console.WriteLine("Asset name: " + asset.Name);
                IFileInfo theFile = asset.Files.FirstOrDefault();

                // You could iterate through the asset.Files collection with a 
                // foreach statement, and download all files for assets that 
                // have multiple files.  
                //  Example: foreach(IFileInfo file in asset.Files)

                // Download the file to the specified local folder.
                if (theFile != null)
                    theFile.DownloadToFile(Path.GetFullPath(outputMediaFilesFolder +
                            Path.DirectorySeparatorChar + theFile.Name));
                else
                    Console.WriteLine("No files available for this asset.");
            }
            else
            {
                Console.WriteLine("Asset not available.");
            }
        }
        static ILocator CreateAssetAssignAccessPolicy(string inputFilePath)
        {
            // Part 1 - Connecting to Azure Media Services
            //          Uploading a video to encode

            CloudMediaContext mediaContext = new CloudMediaContext(
                            new Uri("https://nimbustap.cloudapp.net/API/"),
                            "F7DF85E4-E142-4042-A417-161FE60D409C",
                            "7HCe8R6n",
                            "urn:Nimbus",
                            "https://nimbusvoddev.accesscontrol.windows.net");

            // Create and upload an asset.
            IAsset asset = mediaContext.Assets.Create(inputFilePath);
            // Because this is a single-file asset, get the name of first file. 
            string fileName = asset.Files[0].Name;

            // Create a policy for the asset.
            IAccessPolicy readPolicy = mediaContext.AccessPolicies.Create(
                "CanReadFor30Minutes",
                TimeSpan.FromMinutes(30),
                AccessPermissions.Read);

            // Link access policy to asset
            asset.AccessPolicies.Add(readPolicy);

            // Link access policy to asset
            ILocator locator = asset.Locators.Where(c => c.AccessPolicyId ==
                readPolicy.Id && c.ExpirationDateTime > DateTime.UtcNow).First();

            // Print the path for the locator you created.
            Console.WriteLine("Locator path: " + locator.Path);

            // Get the locator path uri.
            var uriBuilder = new UriBuilder(locator.Path);
            // Build the full path to the file associated with the locator.  
            uriBuilder.Path += Path.AltDirectorySeparatorChar + fileName;
            Uri fullUrl = uriBuilder.Uri;
            // Print the full path to the file. 
            Console.WriteLine("Full URL to file: " + fullUrl);

            return locator;
        }



#elif PART2
        public static CloudMediaContext mediaContext = null;
        static void Main(string[] args)
        {
            CloudMediaContext mediaContext = new CloudMediaContext(
                            new Uri("https://nimbustap.cloudapp.net/API/"),
                            "F7DF85E4-E142-4042-A417-161FE60D409C",
                            "7HCe8R6n",
                            "urn:Nimbus",
                            "https://nimbusvoddev.accesscontrol.windows.net");

            Console.WriteLine("ASSET INFORMATION");
            foreach (IAsset asset in mediaContext.Assets)
            {
                // Display the collection of assets.
                Console.WriteLine("Asset ID: " + asset.Id);
                Console.WriteLine("Name: " + asset.Name);

                // Display the files associated with each asset. 
                foreach (IFileInfo fileItem in asset.Files)
                {
                    Console.WriteLine("File Name: " + fileItem.Name);
                    Console.WriteLine("File Create Date: " + fileItem.Created);
                    Console.WriteLine("File Size: " + fileItem.ContentFileSize);
                }
            }
        }
#elif PART1
        static void Main(string[] args)
        {
            // Part 1 - Connecting to Azure Media Services
            //          Uploading a video to encode
            CloudMediaContext mediaContext = new CloudMediaContext(
                            new Uri("https://nimbustap.cloudapp.net/API/"),
                            "F7DF85E4-E142-4042-A417-161FE60D409C",
                            "7HCe8R6n", 
                            "urn:Nimbus",
                            "https://nimbusvoddev.accesscontrol.windows.net");
            mediaContext.Assets.OnUploadProgress += Assets_OnUploadProgress;

            var asset = mediaContext.Assets.Create(@"C:\windows\Performance\WinSat\winsat.wmv");

            // Part 2 - Create a task, specify encoding details
            Console.Clear();
            var task = mediaContext.Jobs.CreateTask(
                encoder: "Expression Encoder",
                configuration: "H.264 HD 720p VBR",
                inputs: new[] { asset },
                outputs: OutputAsset.CreateSingleOutput());

            // Part 3 - Create a job, display completion progress
            IJob job = mediaContext.Jobs.CreateJob("Sample Job", task);
            while (job.State != JobState.Finished)
            {
                job = mediaContext.Jobs.Where(j => j.Id == job.Id).SingleOrDefault();
                //job = mediaContext.Jobs.Refresh(job.Id);
                Console.SetCursorPosition(0, 0);
                Console.WriteLine("Job Name: " + job.Name);
                Console.WriteLine("Job ID: " + job.Id);
                Console.WriteLine();
                Console.WriteLine("Job State: {0,-20}", job.State);
                Console.WriteLine("Task Progress: {0:0.00}%  ", 
                                        job.Tasks.Single().Progress);

                Thread.Sleep(500);
                mediaContext.Detach(job);
            }

            Console.WriteLine();
            Console.WriteLine("Job Complete!");
            Console.ReadLine();
        }
#endif
        static void performWork()
        {
            CloudMediaContext context = GetContext();

            context.Assets.OnUploadProgress += new EventHandler<UploadProgressEventArgs>(Assets_OnUploadProgress);
            var assetVideo = context.Assets.Create(@"C:\windows\Performance\WinSat\winsat.wmv");
            Console.Clear();

            var task = context.Jobs.CreateTask(
                encoder: "Expression Encoder",
                configuration: "H.264 HD 720p VBR",
                inputs: new[] { assetVideo },
                outputs: OutputAsset.CreateSingleOutput());

            var job = context.Jobs.CreateJob("Sample Job", task);
            while (job.State != JobState.Finished)
            {
                //job = context.Jobs.Refresh(job.Id);
                Console.SetCursorPosition(0, 0);
                Console.WriteLine("Job Name: " + job.Name);
                Console.WriteLine("Job ID: " + job.Id);
                Console.WriteLine();
                Console.WriteLine("Job State: {0,-20}", job.State);
                Console.WriteLine("Task Progress: {0:0.00}%  ", job.Tasks.Single().Progress);

                Thread.Sleep(500);
            }

            Console.WriteLine();
            Console.WriteLine("Job Complete!");
            Console.ReadLine();

        }

        static void Assets_OnUploadProgress(object sender, UploadProgressEventArgs e)
        {
            Console.WriteLine(e.Progress);
        }
        static CloudMediaContext GetContext()
        {

            // Pass in:   Uri apiServer, string clientId, 
            //            string clientSecret, string scope, string acsBaseAddress

            return new CloudMediaContext(new Uri("https://nimbustap1.cloudapp.net/API/"),
                                    "F7DF85E4-E142-4042-A417-161FE60D409C",
                                    "7HCe8R6n",
                                    "urn:Nimbus",
                                    "https://nimbusvoddev.accesscontrol.windows.net");
        }
        //public static IJob Refresh(this JobCollection jobs, string id)
        //{
        //    //mediaContext.DetachAll();
        //    mediaContext.DetachAll();

        //    IJob job = (from j in jobs
        //                where j.Id == id
        //                select j).FirstOrDefault();

        //    return job;
        //}
    }

}

